#!/usr/bin/perl -w

use strict;

# extract.pl
# Extract training examples from CMU Seminars, sequence version
# Fraser

my @tokens = ();
my @classes = ();
my @linebreaks = ();

my $in = 0;

while (<STDIN>) {
    chomp;

    # separate commas, dashes
    s/,/ , /g;
    s/-/ - /g;

    # separate parentheses
    s/\(/ ( /g;
    s/\)/ ) /g;

    # normalize AM/PM
    s/ am / AM /g;
    s/ pm / PM /g;

    s/a\.m\.?/ AM /ig;
    s/p\.m\.?/ PM /ig;

    s/(\d)am /$1 AM /ig;
    s/(\d)pm /$1 PM /ig;

    s/^am /AM /g;
    s/^pm /PM /g;

    # only separate period followed by space or end of line
    s/\.( |$)/ . /g;

    # make sure we tokenize before start tags and after end tags, and that they are flush against a token if possible by removing whitespace
    s/<stime>\s*/ <stime>/g;
    s/\s*<\/stime>/<\/stime> /g;

    my @spacesep = split(/\s+/, $_);

    for (my $i = 0; $i < @spacesep; $i++) {

	# skip it if it is only tags (but not if it has stime or /stime)
	if (not ($spacesep[$i] =~ /<\/?stime>/)) {
	    my $tagfree = $spacesep[$i];
	    $tagfree =~ s/<.*?>//g;
	    if ($tagfree =~ /^\s*$/) {
		next;
	    }
	}

	if ($spacesep[$i] =~ /^<stime>/ and $spacesep[$i] =~ /<\/stime>$/) {

	    #Could instead use label "W-STIME" for "Whole", separating one token entities from multi-token entities

	    push(@classes, "B-STIME");
	    $in = 0;
	}
	elsif ($spacesep[$i] =~ /^<stime>/) {
	    push(@classes, "B-STIME");
	    $in = 1;
	}
	elsif ($spacesep[$i] =~ /<\/stime>$/) {
	    
	    # Could instead use label "E-STIME" for "End"

	    push(@classes, "I-STIME");
	    $in = 0;
	}
	elsif ($in) {
	    push(@classes, "I-STIME");
	}
	else {
	    push(@classes, "O");
	}

	# delete tags
	$spacesep[$i] =~ s/<.*?>//g;

	if ($spacesep[$i] eq /^\s+$/) {
	    push(@tokens, "NULL");
	}
	else {
	    push(@tokens, $spacesep[$i]);
	}

	push(@linebreaks, 0);
    }

    # record a feature for the end of each line (value=1). If we have a following blank line, increment it to 2 (but stop incrementing after that)
    if (@linebreaks > 0 and $linebreaks[@linebreaks-1] < 2) {
	$linebreaks[@linebreaks-1] += 1;
    }
}

# EOF counts as a double blank line
$linebreaks[@linebreaks-1] = 2;


# we have read in the whole file, we have @tokens, @classes and @linebreaks
for (my $i = 0; $i < @tokens; $i++) {

    # Col 0 is current token
    print $tokens[$i], " ";

    # Col 1 is whether current token is a single digit
    if ($tokens[$i] =~ /^\d$/) {
	print "D ";
    }
    else {
	print "NO ";
    }

    # Col 2 is whether current token is a double digit
    if ($tokens[$i] =~ /^\d\d$/) {
	print "DD ";
    }
    else {
	print "NO ";
    }

    # Col 3 is whether current token could look like a time (digit colon digit)
    if ($tokens[$i] =~ /\d:\d/) {
	print "DCD ";
    }
    else {
	print "NO ";
    }

    # Col 4 is whether current token could look like a time (digit colon digit digit)
    if ($tokens[$i] =~ /\d:\d\d/) {
	print "DCDD ";
    }
    else {
	print "NO ";
    }

    # Col 5: if looks like a time, last two digits
    if ($tokens[$i] =~ /\d:(\d\d)/) {
	print "$1 ";
    }
    else {
	print "NO ";
    }

    # Col6: whether we have one or 2+ line breaks after the token (value is 0, 1 or 2)
    print $linebreaks[$i], " ";

    print $classes[$i], "\n";

    # Sequence splitting: we have a new sequence iff we have a period or two linebreaks following
    if ($tokens[$i] eq "." or $linebreaks[$i] == 2) {
	print "\n";
    }
}
